/*
   Module for supplying HasMethod-functionality for checking the superclasses for it

program:   HasMethod_InSuper.rex
type:      Object REXX, REXXSAA 6.0
purpose:   determines whether a given method is available in any of the
           superclasses of the object (cf. comments in code)
version:   1.0.1
date:      1999-05-12
changed:   2005-05-29, ---rgf, renamed ".cmd" to ".rex"

author:    Rony G. Flatscher
           Rony.Flatscher@wu-wien.ac.at
           (Wirtschaftsuniversitaet Wien, University of Economics and Business
           Administration, Vienna/Austria/Europe)

needs:     ---

usage:     call or require this module

comments:  created for the class "Einfhrung in die prozedurale und objektorientierte
           Programmierung (Object Rexx)"


All rights reserved and copyrighted 1999 by the author,
no guarantee that it works without errors, etc. etc.

You are granted the right to use this module under the condition that you don't charge money for this module (as you didn't write
it in the first place) or modules directly derived from this module, that you document the original author (to give appropriate
credit) with the original name of the module and that you make the unaltered, original source-code of this module available on
demand.  If that holds, you may even bundle this module (either in source or compiled form) with commercial software.

If you find an error, please send me the description (preferably a *very* short example);
I'll try to fix it and re-release it to the net.
*/



:: ROUTINE get_methods_FROM_HasMethod_InSuper PUBLIC  /* return the method-directory  */
   RETURN .methods




/* <----------          -----------> */
/* floating method for determining whether self is a class object */
:: METHOD HasMethod_InSuper
   USE ARG Method_Name
   RETURN HasMethod_InSuper( self, Method_Name )


/******************************************************************************/
/*                                                                            */
/* name:    HasMethod_InSuper( object, Method_Name )                          */
/*                                                                            */
/* purpose: determines whether a method with the given Method_Name exists     */
/*          in one of the superclasses                                        */
/*                                                                            */
/* returns: .true if a method by that name exists, .false else                */
/*                                                                            */
/* remarks: ---                                                               */
/*                                                                            */
/*          rgf, 99-05-12                                                     */

/* PUBLIC ROUTINE      */
:: ROUTINE HasMethod_InSuper  PUBLIC
   USE ARG object, meth_name

   class_object = object~class

   RETURN do_the_work( class_object, meth_Name )

/* look thru the superclasses recursively, until method found or at root        */
do_the_work : PROCEDURE
   USE ARG cobj, meth_name

   DO UNTIL cobj = .object
      DO super OVER cobj~SUPERCLASSES
         IF hm_is( super, meth_name) THEN RETURN .true  /* found method in super */

         RETURN do_the_work( super, meth_name )
      END
   END
   RETURN .false                /* arrived at root, method not found    */


::ROUTINE HM_IS
   USE ARG cobj, meth_name

   SIGNAL ON SYNTAX
   tmp = cobj~method( meth_name )       /* try to find method           */
   IF tmp = .nil THEN RETURN .false     /* DEFINEd to be unavailable!   */

   RETURN .true                 /* method available!    */

SYNTAX:                         /* if syntax-error raised, then method not found */
   RETURN .false


/******************************************************************************/




